import pytest
import pickle
from yahoo_ff import yahoo_ff


# TODO add many more tests
#       check if 3 arguments in time in annually, if not, fill with NaN

def test_package_sec_annually():
    aapl = yahoo_ff('aapl')
    aapltest = pickle.load(open('test.p', "rb"))

    for field in aapl.incomestatement_fields:
        assert aapltest.incomestatement_annual[field] == \
               aapl.incomestatement_annual[field]

    for field in aapl.balancesheet_fields:
        assert aapltest.balancesheet_annual[field] == \
               aapl.balancesheet_annual[field]


    for field in aapl.cashflow_fields:
        assert aapltest.cashflow_annual[field] == \
               aapl.cashflow_annual[field]




from .yahoo_ff import yahoo_ff
import os
import pickle


# TODO check if exists already, check if all files are there

class stocks_database:
    '''creates a pickle database for a list of tickers specified in list.format file'''
    format = '.csv'

    def __init__(self, list):

        self.failed = []
        self.name = list
        self.filename = list + stocks_database.format # which file do you read to construct it
        self.location = os.getcwd() + '/' + list + '_db/'
        if not os.path.exists(self.location):
            os.makedirs(self.location)
            print 'created database folder ' + self.location
            self.__create()
        else:
            print 'database ' + self.name + ' already exists at ' + self.location
            # TODO function to update simply and quickly
        print 'failed to store data for these stocks : ', self.failed

    def __create(self):
        '''create the database with for loop, make pickle file for each ticker'''
        with open(self.filename, 'r') as f:
            tickers = (f.read().split())
            for ticker in tickers:
                # if not in database add pickle file
                if not os.path.exists(self.location + ticker + '.p'):
                    data = yahoo_ff(ticker)
                    # if no flag proceed
                    if not data.flag:
                        pickle.dump(data, open(self.location + ticker + '.p', 'wb'))
                        print 'dumped to pickle data of ' + ticker
                    else:
                        print 'failed to get full data of ' + ticker
                        self.failed.append(ticker)
                # if already in database, move on to the next ticker
                else:
                    print 'data for ' + ticker + ' already in database ' + self.name

    def take(self, ticker):
        '''return the yahoo_ff object that was stored in a pickle file of the database'''
        if os.path.exists(self.location + ticker + '.p'):
            return pickle.load(open(self.location + ticker + '.p', 'rb'))
        else:
            print 'no record of ' + ticker + ' in database ' + self.name
            return None



import urllib2


def parse_powers(x):
    powers = {'B': 10 ** 9, 'M': 10 ** 6, 'T': 10 ** 12}
    try:
        power = x[-1]
        return float(x[:-1]) * powers[power]
    except TypeError:
        return x


def float_or_none(x):
     x = x.replace(',','')
     try:
         # if negative value (1000)
         if x[0]=='(' and x[-1]==')':
            return -float(x[1:-2])
         else:
             return float(x)
     except: return None


def request(source_code, information):
    return  parse_table(
            find_section(source_code,information))


def get_annual_incomestatement_url(stock):
    return 'https://finance.yahoo.com/q/is?s='+stock+'&annual'


def get_quarterly_incomestatement_url(stock):
    return 'https://finance.yahoo.com/q/is?s='+stock


def get_annual_balancesheet_url(stock):
    return 'https://finance.yahoo.com/q/bs?s='+stock+'&annual'


def get_quarterly_balancesheet_url(stock):
    return 'https://finance.yahoo.com/q/bs?s='+stock


def get_annual_cashflow_url(stock):
    return 'https://finance.yahoo.com/q/cf?s='+stock+'&annual'


def get_quarterly_cashflow_url(stock):
    return 'https://finance.yahoo.com/q/cf?s='+stock


def get_stockinfo_url(stock):
    return 'http://finance.yahoo.com/q/pr?s='+stock+'+Profile'


def get_keystats_url(stock):
    return 'http://finance.yahoo.com/q/ks?s=' + stock


def get_source_code(url):
    return urllib2.urlopen(url).read()


def parse_table(source_code):
    source_code = source_code.split('</td></tr>')[0]
    source_code = source_code.replace('<strong>', '')
    source_code = source_code.replace('</strong>', '')
    source_code = source_code.replace('\n', '')
    source_code = source_code.replace('&nbsp;', '')
    source_code = source_code.replace('<td align="right">','')
    source_code = source_code.replace(' ', '')
    source_code = source_code.split('</td>')
    source_code = filter(None, source_code)
    return [float_or_none(x.replace(',', '')) for x in source_code]


def find_section(source_code, section_name):
    try:
        return source_code.split(section_name)[1]
    except:
        print 'failed acquiring ' + section_name


def get_infos(source_code):
    sector = source_code.split('Sector:')[1].split('</td>')[1].replace('</a>','').split('>')[-1]
    industry = source_code.split('Industry:')[1].split('</td>')[1].replace('</a>','').split('>')[-1]
    employees = source_code.split('Full Time Employees:')[1].split('</td>')[1].replace('</a>',
                                                                                    '').split('>')[-1]
    return {'sector' : sector, 'industry' : industry, 'full time employees': float_or_none(employees)}


def get_keystats(source_code):
    sharesout = parse_powers(source_code.split('Shares Outstanding')[1].split('</td></tr>')[
                                 0].split('>')[-1])
    return {'sharesout': sharesout}


import time
import json

from dateutil import parser
import Quandl
import pandas as pd
import constants

from .tools import *


class yahoo_ff:
    '''class contains incomestatement, balancesheet, cashflow and price history for a ticker'''
    incomestatement_fields = constants.incomestatement_fields
    balancesheet_fields = constants.balancesheet_fields
    cashflow_fields = constants.cashflow_fields
    sleep = 1 # sec

    def __init__(self, ticker):
            self.flag = 0
            self.ticker = ticker
            self.__wait()
            self.__construct_incomestatement_annual()
            self.__wait()
            self.__construct_incomestatement_quarterly()
            self.__wait()
            self.__construct_balancesheet_annual()
            self.__wait()
            self.__construct_balancesheet_quarterly()
            self.__wait()
            self.__construct_cashflow_annual()
            self.__wait()
            self.__construct_cashflow_quarterly()
            self.__wait()
            self.__construct_stockinfo()
            self.__wait()
            self.__get_pricehistory()
            print 'flag is ' + str(self.flag)

    def __construct_incomestatement_annual(self):
        '''populate self.incomestatement_annual'''
        try:
            html = get_source_code(
                get_annual_incomestatement_url(
                    self.ticker)).split('Get Income Statement for:')[1]

            self.incomestatement_annual = self.__get_endofperiod(html)
            for field in self.incomestatement_fields:
                self.incomestatement_annual[field] = request(html, field)
            print 'Annual income statement for ' + str(self.ticker) + ' successfuly obtained'
        except Exception,e:
            self.flag = 1
            print 'failed construct_incomestatement_annual for ' + self.ticker + '; ' + str(e)

    def __construct_incomestatement_quarterly(self):
        '''populate self.incomestatement_quarterly'''
        try:
            html = get_source_code(
                get_quarterly_incomestatement_url(
                    self.ticker)).split('Get Income Statement for:')[1]

            self.incomestatement_quarterly = self.__get_endofperiod(html)
            for field in self.incomestatement_fields:
                self.incomestatement_quarterly[field] = request(html, field)
            print 'Quarterly income statement for ' + str(self.ticker) + ' successfuly obtained'
        except Exception, e:
            self.flag = 1
            print 'failed construct_incomestatement_quarterly for ' + self.ticker + '; ' + str(e)

    def __construct_balancesheet_annual(self):
        '''populate self.balancesheet_annual'''
        try:
            html = get_source_code(
                get_annual_balancesheet_url(
                    self.ticker)).split('Get Balance Sheet for:')[1]
            self.balancesheet_annual = self.__get_endofperiod(html)
            for field in self.balancesheet_fields:
                self.balancesheet_annual[field] = request(html, field)
            print 'Annual balance sheet for ' + str(self.ticker) + ' successfuly obtained'
        except Exception, e:
            self.flag = 1
            print 'failed construct_balancesheet_annual for ' + self.ticker + '; ' + str(e)

    def __construct_balancesheet_quarterly(self):
        '''populate self.balancesheet_quarterly'''
        try:
            html = get_source_code(
            get_quarterly_balancesheet_url(
                self.ticker)).split('Get Balance Sheet for:')[1]
            self.balancesheet_quarterly = self.__get_endofperiod(html)
            for field in self.balancesheet_fields:
                self.balancesheet_quarterly[field] = request(html, field)
            print 'Quarterly balance sheet for ' + str(self.ticker) + ' successfuly obtained'
        except Exception, e:
            self.flag = 1
            print 'failed construct_balancesheet_quarterly for ' + self.ticker + '; ' + str(e)

    def __construct_cashflow_annual(self):
        '''populate self.cashflow_annual'''
        try:
            html = get_source_code(
                get_annual_cashflow_url(
                    self.ticker)).split('Get Cash Flow for:')[1]
            self.cashflow_annual = self.__get_endofperiod(html)
            for field in self.cashflow_fields:
                self.cashflow_annual[field] = request(html, field)
            print 'Annual Cash Flows for ' + str(self.ticker) + ' successfuly obtained'
        except Exception, e:
            self.flag = 1
            print 'failed construct_cashflow_annual for ' + self.ticker + '; ' + str(e)

    def __construct_cashflow_quarterly(self):
        '''populate self.cashflow_quarterly'''
        try:
            html = get_source_code(
                get_quarterly_cashflow_url(
                    self.ticker)).split('Get Cash Flow for:')[1]
            self.cashflow_quarterly = self.__get_endofperiod(html)
            for field in self.cashflow_fields:
                self.cashflow_quarterly[field] = request(html, field)
            print 'Quarterly Cash Flows for ' + str(self.ticker) + ' successfuly obtained'
        except Exception, e:
            self.flag = 1
            print 'failed construct_cashflow_quarterly for ' + self.ticker + '; ' + str(e)

    def __construct_stockinfo(self):
        '''get basic information on the stock (sector, industry, nb of employees'''
        try:
            html = get_source_code(get_stockinfo_url(self.ticker))
            self.infos = get_infos(html)
            html = get_source_code(get_keystats_url(self.ticker))
            self.keystats = get_keystats(html)

        except Exception, e:
            self.flag = 1
            print 'failed construct_stockinfo for ' + self.ticker + '; ' + str(e)

    def __get_endofperiod(self, html):
        '''scrape the html source code for the ending periods of each column'''
        source_code = html
        end_periods = source_code.split('Period Ending')[1]
        end_periods = end_periods.split('</TR>')[0]
        # take out unwanted html formatting
        end_periods = end_periods.replace(
            '<TD class="yfnc_modtitle1" align="right"><b>', '')
        end_periods = end_periods.replace('<th scope="col" style="border-top:2px solid '
                                          '#000;text-align:right; font-weight:bold">', '')
        end_periods = end_periods.replace('</span></small></td>', '')
        end_periods = end_periods.replace('</span></small></TD>', '')
        end_periods = end_periods.replace('</b>', '')
        #s split
        end_periods = end_periods.split('</th>')
        # if '</th>' is not used to split periods
        if len(end_periods) == 1:
            end_periods = end_periods[0].split('</TD>')

        return {'endofperiod': [parser.parse(x[-constants.date_string_length:]) for x in
                                end_periods if x is not '']}

    def __get_pricehistory(self):
        '''get stock price history and volume traded using quandl api'''
        with open('credentials.json', 'r') as creds:
            credentials = json.load(creds)
            try:
                self.pricehistory = Quandl.get(
                    'WIKI/' + self.ticker, authtoken=credentials['Quandl']['key'])
            except Exception, e:
                self.flag = 1
                print 'failed get_pricehistory for ' + self.ticker + '; ' + str(e)

    def __wait(self):
        time.sleep(yahoo_ff.sleep)

        def package_sec_annually(self):
            '''package all annual info from incomestatement, blanacesheet and cashflow into a pandas
            dataframe'''
            isa = pd.DataFrame(self.incomestatement_annual)
            bsa = pd.DataFrame(self.balancesheet_annual)
            csa = pd.DataFrame(self.cashflow_annual)
            df = pd.merge(isa, bsa, on='endofperiod')
            df1 = pd.merge(df, csa, on='endofperiod')
            df1.set_index('endofperiod', inplace=True)
            df1 = df1.transpose()
            return df1

        def package_sec_quarterly(self):
            '''package all quarterly info from incomestatement, blanacesheet and cashflow into a pandas
            dataframe'''
            isa = pd.DataFrame(self.incomestatement_quarterly)
            bsa = pd.DataFrame(self.balancesheet_quarterly)
            csa = pd.DataFrame(self.cashflow_quarterly)
            df = pd.merge(isa, bsa, on='endofperiod')
            df1 = pd.merge(df, csa, on='endofperiod')
            df1.set_index('endofperiod', inplace=True)
            df1 = df1.transpose()
            return df1

    def package_sec_annually(self):
        '''package all annual info from incomestatement, blanacesheet and cashflow into a pandas
        dataframe'''
        isa = pd.DataFrame(self.incomestatement_annual)
        bsa = pd.DataFrame(self.balancesheet_annual)
        csa = pd.DataFrame(self.cashflow_annual)
        df = pd.merge(isa, bsa, on='endofperiod')
        df1 = pd.merge(df, csa, on='endofperiod')
        df1.set_index('endofperiod', inplace=True)
        df1 = df1.transpose()
        return df1

    def package_sec_quarterly(self):
        '''package all quarterly info from incomestatement, blanacesheet and cashflow into a pandas
        dataframe'''
        isa = pd.DataFrame(self.incomestatement_quarterly)
        bsa = pd.DataFrame(self.balancesheet_quarterly)
        csa = pd.DataFrame(self.cashflow_quarterly)
        df = pd.merge(isa, bsa, on='endofperiod')
        df1 = pd.merge(df, csa, on='endofperiod')
        df1.set_index('endofperiod', inplace=True)
        df1 = df1.transpose()
        return df1



from setuptools import setup

setup(name='yahoo_ff',
      version='0.13',
      description='retrieve financial fundamental data for stocks',
      url='http://github.com/alexandresobolevski/yahoo_ff',
      author='Alexandre Sobolevski',
      author_email='',
      license='MIT',
      packages=['yahoo_ff'],
      zip_safe=False)
